/** This code is under the GPL, viewable at http://www.gnu.org/copyleft/gpl.html
 *  You are free to modify and redistribute the code as you see fit, but three main 
 *  conditions exist for usage of this code:
 *  1.  You may only pass on this code with the same rights and priveleges that were given
 *      to you (In other words, you must redistribute this license)
 *  2.  You may NOT distribute this code for sale or for profit.
 *  3.  There are no warranties about this code.  It is designed not to screw up your computer,
 *      but in the case that it does, the writer cannot be held responsible for losses.
 *
 *  Code author:  flug_m (post comments/questions to http://www.saveourtunes.com/forum/ In future
 *                versions, I'll make available alternative means of contact.)
 *  Code Revision date:  September 1, 2007
 *  Code Revision version: 0.5
 *
 *  The GUI class launches the program.
 *
 *  Changes:
 *  8/30/07  The configuration file is created from this class.  It also creates the thread that
 *           saves the config settings at program death.
 *  9/03/07  Uses system look and feel.
 *  3/26/08  On Mac OSX, the menu bar is now placed in the Screen Menu Bar.
 */
 package ourtunes.gui;
 
import javax.swing.*;
import javax.swing.border.*;
import javax.jmdns.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Properties;

import ourtunes.net.*;
import ourtunes.Shared;
import ourtunes.os.OS;

public class GUI
{
	public static void main(String[] args)
	{
		System.setProperty("apple.laf.useScreenMenuBar", "true");

		OurTunesFrame o = new OurTunesFrame();
		o.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		JmDNS bonjour = Shared.getJmDNSInstance();
		
		if(bonjour == null) System.exit(1);
		
		OurTunesPanel op = new OurTunesPanel();
		
		
		//create the service listener responsible for reporting .local iTunes Libraries.
		daapService daap = new daapService(op);
		o.addPanel(op);
		bonjour.addServiceListener("_daap._tcp.local.",daap);
		
		o.setVisible(true);
		System.out.println(bonjour.getHostName());
		
		//create the Host thread responsible for listening with the pass-through.
		Host h = null;
		try{
		h = new Host();
		h.start();
		} catch(java.io.IOException e)
		{
			JOptionPane.showMessageDialog(o, "An error occurred while launching the iTunesHost "+
				"thread.\nLaunch ourTunes from the console or terminal for more info.");
			e.printStackTrace();
		}
		
		//Create the Properties object which manages the preferences for ourTunes.
		
		OS opSys = Shared.getOS();
		Properties p = opSys.loadConfiguration();
		
		if (p == null)
		{
			//We need to create the properties file
			p = new Properties();
			p.setProperty("home" , opSys.getDefaultDirectory());
			p.setProperty("format", op.getFmtText());
			p.setProperty("counter", "1");
		}
		else
		{
			//We need to update the fields from config file
			op.setFmtText(p.getProperty("format"));
		}
		//set the properties object in the Shared class
		Shared.setConfiguration(p);
		
		//Set the ourTunes panel in the Shared class
		Shared.panel = op;
		
		//At program closing, run a thread to save the configuration settings.
		Runtime.getRuntime().addShutdownHook(new ourtunes.SaveConfig());
		
		//Use the system look and feel
		//opSys.changeLookAndFeel();
			
		//Debug:  watch the threads presently running
		//java.util.Timer timer = new java.util.Timer();
		//timer.scheduleAtFixedRate(new listThreads(),3000,6000);
	}
}

//This class is primarily for debugging.  
//Use this with a java.util.Timer, and not a javax.swing.Timer
//They are two different classes!)
class listThreads extends java.util.TimerTask
{
	public void run()
	{
		int count = Thread.activeCount();
		
		Thread[] arr = new Thread[count];
		
		Thread.enumerate(arr);
		System.out.println("------------------------------------");
		for (Thread t : arr)
			System.out.println("\t"+t+" Active:  "+t.isAlive());
	}
}