/** This code is under the GPL, viewable at http://www.gnu.org/copyleft/gpl.html
 *  You are free to modify and redistribute the code as you see fit, but three main 
 *  conditions exist for usage of this code:
 *  1.  You may only pass on this code with the same rights and priveleges that were given
 *      to you (In other words, you must redistribute this license)
 *  2.  You may NOT distribute this code for sale or for profit.
 *  3.  There are no warranties about this code.  It is designed not to screw up your computer,
 *      but in the case that it does, the writer cannot be held responsible for losses.
 *
 *  Code author:  flug_m (post comments/questions to http://www.saveourtunes.com/forum/ In future
 *                versions, I'll make available alternative means of contact.)
 *  Code Revision date:  March 26, 2008
 *  Code Revision version: 0.7
 *
 *  The ListListener is responsible for coordinating the daapService class
 *  and the GUI list of iTunes libraries.  It shouldn't change much
 *  from its current form.
 *
 *  Changes:
 *  8/31/07  minor changes to html formatted strings.
 *  9/06/07  We leave an InetAddress in the Shared class instead of a string (with server name).
 * 11/30/07  Update to prevent an exception on the class.  An update to ourTunesPanel affected
 *              how the listener responds to events.
 *  3/26/08  Due to changes between v1 and v2 of the JmDNS library, all code referencing the library was
 *           rewritten to correctly invoke the library.  
 *  7/23/08  Changed a port number: ourTunes no longer pirates the port away from iTunes.
 *  7/23/08  Minor change to the text string for Windows OS
 * 10/21/08  Changed a method call from replace() to replaceFirst() to change the machine name value
 *           Insert:  first run property, set when a new bonjour service is registered.
 *
 */
 package ourtunes.gui;
 
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.jmdns.*;
import javax.swing.*;
import java.io.IOException;

import ourtunes.Shared;
import ourtunes.net.*;

public class ListListener implements ListSelectionListener
{
	public ListListener(OurTunesPanel panel)
	{
		this.panel = panel;
	}
	
	public void valueChanged(ListSelectionEvent event)
	{
		if (event.getValueIsAdjusting()) {System.out.println("Value changed"); panel.acknowledgeListChanged();
				return;}
		if (panel.getListChanged()) { System.out.println("Caught on list-changed");
			panel.acknowledgeListChanged(); return; }
		
		final JmDNS bonjour = Shared.getJmDNSInstance();
		bonjour.unregisterAllServices();
		
		//get the element selected.
		JList dlist = (JList) event.getSource();
		final ListItem item = (ListItem) dlist.getSelectedValue();
		if (item == null) return;
		//request the service for hosting our service with mDNS.
		Runnable r = new Runnable(){
			public void run()
			{
			panel.setStatus("<code style=\"color:red\"> Creating pass-through thread...</code>");
			ServiceInfo old = 	bonjour.getServiceInfo("_daap._tcp.local.", item.toString());
			Shared.setServer(old.getAddress());		
			//Because this was called in response to a selection,
			//we should register the service in mDNS and
			//update the status pane.
			String text = old.getTextString();
			//if (Shared.getOS() instanceof ourtunes.os.Windows) text = text.replaceAll("\n","\r\n");
			text = text.replaceFirst("\\QMachine Name=\\E","Machine Name=+");
			ServiceInfo info = ServiceInfo.create("_daap._tcp.local.","Tunnel",3688,text); 
		      Shared.setProperty("port.FirstOpen", "1");
			try{
				bonjour.registerService(info);
			}catch(IOException ex)
			{
				ex.printStackTrace();
				JOptionPane.showMessageDialog(panel, "An error occurred registering the Tunnel.\nRun ourTunes from"+
					" the console or terminal to see more details.");
			}
		
			//...and update the status
			String status = "<P STYLE=\"color: red;\">**Note:  Do not select another library<br>"+
					"while connected to this one.**</p><p>";
			
			status = status + Shared.getHTMLText(info) + "</P>";			
			panel.setStatus(status);
			
			}
		};
		Thread t = new Thread(r, "valueChanged-serviceInfo");
		t.start();
	}
	
	private OurTunesPanel panel;
}
