/** This code is under the GPL, viewable at http://www.gnu.org/copyleft/gpl.html
 *  You are free to modify and redistribute the code as you see fit, but three main 
 *  conditions exist for usage of this code:
 *  1.  You may only pass on this code with the same rights and priveleges that were given
 *      to you (In other words, you must redistribute this license)
 *  2.  You may NOT distribute this code for sale or for profit.
 *  3.  There are no warranties about this code.  It is designed not to screw up your computer,
 *      but in the case that it does, the writer cannot be held responsible for losses.
 *
 *  Code author:  flug_m (post comments/questions to http://www.saveourtunes.com/forum/ In future
 *                versions, I'll make available alternative means of contact.)
 *  Code Revision date:  February 22, 2008
 *  Code Revision version: 0.7
 *
 *  This panel organizes and presents all the GUI elements to the user.
 *  
 *  Changes:
 *  8/30/07  The status pane consists of a JEditorPane instead of a JLabel.
 *  8/30/07  Addition of accessor methods for the format/status panes (not documented).
 *  9/03/07  Icon embedded into the Panel.  It should appear just above the status pane.
 * 11/28/07  appendDownloadStatus(int,String) is synchronized to help alleviate thread lock.
 * 11/28/07  addtoList() and removeFromList() are marked as synchronized.
 * 11/30/07  addtoList() and removeFromList() prevent new service discoveries/removals from affecting
 *                       the currently selected value.
 * 11/30/07  removeFromList() clears the selected library if the selected library is being removed.
 *  2/22/08  Reentrant Locks are used to control access to the GUI elements instead of through
 *                       synchornization.  The latter method is slowing the program considerably,
 *                       often causing iTunes to lock up.  Changed Methods (and corresponding Locks):
 *                       1.  rllist - addToList() & removeFromList()
 *                       2.  rlstatus - appendDownloadStatus(), setStatus(), lockStatus(), isStatusAvailable(),
 *                                      lockStatusGUI()
 */
 package ourtunes.gui;
 
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.util.concurrent.*;
import java.util.concurrent.locks.*;

import ourtunes.Shared;
import ourtunes.net.*;

public class OurTunesPanel extends JPanel
{
	private JTextField fmt;
	private JList list;
	private JEditorPane status;
	private ReentrantLock rllist;
	private ReentrantLock rlstatus;	
	
	public OurTunesPanel()
	{
		//This panel is the main panel; we'll lay out several panels inside of it.
		JPanel leftPanel = buildLeftPanel();
		JPanel topRight = buildTopPanel();
		JPanel bottomRight = buildBottomPanel();
		rlstatus = new ReentrantLock();
		rllist = new ReentrantLock(true);
	
		setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		add(leftPanel);
		Box b = Box.createVerticalBox();
		b.add(topRight);
		b.add(bottomRight);
		add(b);
		
		image = null;
	}
	
	private JPanel buildLeftPanel()
	{
		JPanel p = new JPanel();
		String name = new String("List of iTunes Clients");
		list = new JList();
		DefaultListModel dlm = new DefaultListModel();
		list.setModel(dlm);
		list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		//For me, I'm building, so I'll put in some dummy items to test this container
				
		p.setLayout(new BorderLayout());
		
		//Create a line border with rounded corners.
		p.add(new JScrollPane(list, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER),
			BorderLayout.CENTER);
		TitledBorder b = BorderFactory.createTitledBorder(new LineBorder(Color.GREEN, 1,true),
			name, TitledBorder.LEFT, TitledBorder.TOP, 
			new Font("Arial", Font.ITALIC|Font.BOLD, 12));
		
		//Add the list listener
		list.addListSelectionListener(new ListListener(this));
		
		p.setBorder(b);	
		return p;
	}
	
	private JPanel buildTopPanel()
	{
		//This Panel contains the formatting information.
		String name = new String("Format outputted file");
		String supportedFormats = new String("%r - random Number (4 digits); %R - sequential number (4 digits);<br>"+
			"%a - artist; %t - title; %e - file extension");
			
		JLabel formats = new JLabel("<HTML><BODY>"+supportedFormats+"</HTML>");
		fmt = new JTextField("%R_%a-%t.%e",25);
		JPanel p = new JPanel();
		
		fmt.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent event)
			{
				Shared.setProperty("format", fmt.getText());
				System.out.println("format value changed...");
			}
		});		
		
		p.add(fmt);
		p.add(formats);
		
		TitledBorder tb = BorderFactory.createTitledBorder(new LineBorder(Color.GREEN, 1,true),
			name, TitledBorder.LEFT, TitledBorder.TOP, 
			new Font("Arial", Font.ITALIC|Font.BOLD, 12));
		
		p.setBorder(tb);	
		return p;
	}
	private JPanel buildBottomPanel()
	{
		//This panel contains status information.
		JPanel p = new JPanel();
		status = new JEditorPane("text/html", "<HTML><BODY><p>ourTunes initialized<br></p>" );
		JScrollPane sp = new JScrollPane(status, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
		   JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		p.setLayout(new GridLayout(1,1));
		p.add (sp);
		
		TitledBorder b = BorderFactory.createTitledBorder(new LineBorder(Color.GREEN, 1,true),
			"Status", TitledBorder.LEFT, TitledBorder.TOP, 
			new Font("Arial", Font.ITALIC|Font.BOLD, 12));
		
		p.setBorder(b);	
		return p;
	}
	public void addtoList(ListItem item)
	{
		//Lock the list while it is being modified.
		rllist.lock();
	
		try{
		listhelper = true;
		list.setValueIsAdjusting(true);
		ListModel lmodel = list.getModel();
		DefaultListModel lm = (DefaultListModel) lmodel;
		
		//code copy from com.strangeberry.jmdns.tools.Browser
		for (int i = 0, n = lm.getSize() ; i < n ; i++) {
            if (item.toString().compareToIgnoreCase(lm.elementAt(i).toString()) < 0) {
                lm.insertElementAt(item, i);
                //list.setValueIsAdjusting(false);
                return;
            }
        }
        lm.addElement(item);
        //list.setValueIsAdjusting(false);
        Thread.sleep(150);
		} catch(Exception e) {e.printStackTrace();} finally {rllist.unlock();}
	}
	
	public void removeFromList(ListItem item)
	{
		//Lock the list while it is being modified
		rllist.lock();
		
		try{
		listhelper = true;
		list.setValueIsAdjusting(true);
		ListModel lmodel = list.getModel();
		DefaultListModel lm = (DefaultListModel) lmodel;
		
		//ensure that the selected library deselects properly.
		ListItem li = (ListItem) list.getSelectedValue();
		if (li != null && li.toString().equalsIgnoreCase(item.toString()))
		{
			setStatus("<p style=\"color:red\">The library you were connected to is no longer available"+
					" on the network.</p>");
			list.clearSelection();
		}
		//code copy from com.strangeberry.jmdns.tools.Browser
		for (int i = 0, n = lm.getSize() ; i < n ; i++) {
            if (item.toString().compareToIgnoreCase(lm.elementAt(i).toString()) == 0) {
                lm.removeElementAt(i);
                //list.setValueIsAdjusting(false);
                break;
            }
        }
        Thread.sleep(150);
		}catch(Exception e) {e.printStackTrace();} finally {rllist.unlock();}
        //list.setValueIsAdjusting(false);
	}
	/**
	 *Paint an image into the panel.  The goal is to paint it near the center of the panel.
	 */
	 public void paint(Graphics g)
	 {
	 	super.paint(g);
	 	Dimension d = getSize();
	 	
	 	if (image == null)
	 	{
	 	ImageIcon img = new ImageIcon(ourtunes.about.ObtainIcon.getIcon());
	 	image = img.getImage().getScaledInstance(50,50,Image.SCALE_AREA_AVERAGING);
	 	}
	 	d.width /= 3;
	 	d.height /=3;
	 	g.drawImage(image, d.width, d.height, null);
	 }
	
	
	//Accessor methods for working with the fmt field and the status pane.
	public void setStatus(String text)
	{
		rlstatus.lock();

		try{
		status.setText("<HTML><BODY>"+text+"<br>");
		repaint();
		} finally {rlstatus.unlock();}
		return;	
	}
	
	public void appendDownloadStatus(int contentLength, String ext)
	{
		//Ensure that the thread holds the lock on the Status GUI before
		//permitting updates.
		if (!rlstatus.isHeldByCurrentThread())	return;
		/*
		 *This method carries a little intelligence with it so I don't
		 *have to calculate the values elsewhere.
		 *
		 *As a minor bug, if an interrupt of the pass-through thread occurs, the
		 *maxContentLength variable isn't reset.  However, when the status panel
		 *reads 0% remaining, it will always mean 0% remaining.
	 	*/
	 	if (contentLength <= 0)
		{
			setStatus("Download complete.");
			maxContentLength = -1;
		}
		else if (contentLength > maxContentLength)
		{
			maxContentLength = contentLength;
			setStatus("New Download... 100% remaining");
		}
		else
		{
			double ratio = (double) (contentLength*100.0/maxContentLength);
			contentLength = (int) ratio;
			setStatus("Downloading "+ext+"... "+contentLength+"% remaining...");
		}
	 
	}
	public void appendDownloadStatus(int contentLength)
	{
		//locking is taken care of in method below.  A call
		//to this method may be ignored if the thread does
		//not own the lock.
		appendDownloadStatus(contentLength, "");
	}
	public String getFmtText()
	{
		return fmt.getText();
	}
	public void setFmtText(String text)
	{
		fmt.setText(text);
	}
	public boolean getListChanged()
	{
		return listhelper;
	}
	public void acknowledgeListChanged()
	{
		listhelper = false;
	}
	
	public boolean isStatusAvailable()
	{
		boolean result = rlstatus.tryLock();
		if (result) rlstatus.unlock();
		return result;
	}
	public void lock()
	{
		//Have the thread acquire the lock for the status
		rlstatus.lock();
		return;
	}
	public void unlock()
	{
		rlstatus.unlock();
		return;
	}
	
	private int maxContentLength = -1;
	private Image image;
	private boolean listhelper = false;		
}