/** This code is under the GPL, viewable at http://www.gnu.org/copyleft/gpl.html
 *  You are free to modify and redistribute the code as you see fit, but three main 
 *  conditions exist for usage of this code:
 *  1.  You may only pass on this code with the same rights and priveleges that were given
 *      to you (In other words, you must redistribute this license)
 *  2.  You may NOT distribute this code for sale or for profit.
 *  3.  There are no warranties about this code.  It is designed not to screw up your computer,
 *      but in the case that it does, the writer cannot be held responsible for losses.
 *
 *  Code author:  flug_m (post comments/questions to http://www.saveourtunes.com/forum/ In future
 *                versions, I'll make available alternative means of contact.)
 *  Code Revision date:  November 17, 2007
 *  Code Revision version: 0.5
 *
 * In an effort to self tag files, the AAC class provides the means to interact with Apple's
 * proprietary AAC audio file format.  AAC encoded audio is represented by the following
 * file types:
 *     1.  M4A - AAC encoded audio
 *     2.  M4P - AAC encoded audio protected with DRM.
 *
 * Note that ourTunes does not remove copy protection; that is left to the Jhymn project.
 * However, other developers may use the GPL to integrate both projects. 
 *
 */
 
package ourtunes.music;

import org.jaudiotagger.audio.*;
import org.jaudiotagger.tag.*;
import java.io.File;

public class AAC implements Song
{
	public AAC(File f)
	{
		
		try{
			AudioFile af = 	AudioFileIO.read(f);
			t = af.getTag();
			af = null;
		} catch (Exception e) {
			System.out.println("Problem reading Audio File!");
			e.printStackTrace();
			t = null;
		}
	}
	public String getTitle()
	{
		if (t != null) 	return t.getFirstTitle();
		else return "";
	}
	public String getArtist()
	{
		if (t != null) 	return t.getFirstArtist();
		else return "";
	}
	public int getYear()
	{
		if (t != null)
		{
			String y = t.getFirstYear();
			return Integer.parseInt(y);
		}
		else return -1;
	}
	public String getGenre()
	{
		if (t != null)
			return t.getFirstGenre();
		else return "";
	}
	public String toString()
	{
		return new String("[AAC Title: "+getTitle()+"::Artist: "+getArtist()+"]");
	}
	public void close()
	{
		
	}
	
	private Tag t;
}