/** This code is under the GPL, viewable at http://www.gnu.org/copyleft/gpl.html
 *  You are free to modify and redistribute the code as you see fit, but three main 
 *  conditions exist for usage of this code:
 *  1.  You may only pass on this code with the same rights and priveleges that were given
 *      to you (In other words, you must redistribute this license)
 *  2.  You may NOT distribute this code for sale or for profit.
 *  3.  There are no warranties about this code.  It is designed not to screw up your computer,
 *      but in the case that it does, the writer cannot be held responsible for losses.
 *
 *  Code author:  flug_m (post comments/questions to http://www.saveourtunes.com/forum/ In future
 *                versions, I'll make available alternative means of contact.)
 *  Code Revision date:  July 23, 2008
 *  Code Revision version: 1.0 (final)
 *
 *  This class is added to support files which do not contain meta information inside the file;
 *  in particular, two types of files would be midi files and wav files.
 *
 *  In this case, the methods will return strings that are relevant to the user, but a call to getString() will
 *  return the string stored within the constructor.
 *
 */
package ourtunes.music;
import java.io.File;

public class GenericSong implements Song{

    public GenericSong(File f, String info) 
    {
    	this.info = info;
    	this.f = f;
    }
    /**
     *Return the year this generic file was obtained.  It always asks for the current
     *year as provided by the operating System.
     *
     *@Returns: an int, the current year
     */
    public int getYear()
    {
    	java.util.Calendar c= java.util.Calendar.getInstance();
    	return c.get(c.YEAR);
    }
    /**
     *Return the user who downloaded the generic song over ourTunes. Essentially,
     *this is a call for a system property, which is the best guess of ourTunes. 
     *
     *@Returns:  the user who downloaded the song.  
     */
    public String getArtist()
    {
    	return System.getProperty("user.name");
    }
    /**
     *Although this method would otherwise return the title of the song downloaded, 
     *that info is not contained within the metadata of the song and cannot therefore
     *be given through this interface.  Therefore, this method returns an empty string.
     *
     *@Returns:  an empty string.
     */
    public String getTitle()
    {
    	return "";
    }
    /**
     *Although this method would otherwise return the genre of the song downloaded, 
     *that info is not contained within the metadata of the song and cannot therefore
     *be given through this interface.  Therefore, this method returns an empty string.
     *
     *@Returns:  an empty string.
     */
    public String getGenre()
    {
    	return "";
    }
    
    /**
     *This method returns the string given from the constructor as to minimize the 
     *the difference between the intended effect of the interface and the data
     *this class can provide.
     *
     *@Returns: a string.  In particular, the string given in the constructor.  
     */
     public String toString()
     {
     	return info;
     }
     
     /**
      *Close does nothing.
      */
     public void close() {}
    private String info;
    private File f;
    
}