/** This code is under the GPL, viewable at http://www.gnu.org/copyleft/gpl.html
 *  You are free to modify and redistribute the code as you see fit, but three main 
 *  conditions exist for usage of this code:
 *  1.  You may only pass on this code with the same rights and priveleges that were given
 *      to you (In other words, you must redistribute this license)
 *  2.  You may NOT distribute this code for sale or for profit.
 *  3.  There are no warranties about this code.  It is designed not to screw up your computer,
 *      but in the case that it does, the writer cannot be held responsible for losses.
 *
 *  Code author:  flug_m (post comments/questions to http://www.saveourtunes.com/forum/ In future
 *                versions, I'll make available alternative means of contact.)
 *  Code Revision date:  January 23, 2008
 *  Code Revision version: 0.7
 *
 * This is a wrapper class for the byte[].  The primary reason for its development
 * was to overcome the java Generics problem (I have no cluse if byte[] is acceptable
 * as a generics paramter), but after a while, it contained enough utility methods
 * to make it worthwhile.
 *
 * Changelog 
 *  1/23/2008   The ByteArray has an optional parameter for identification purposes
 *
 */
 package ourtunes.net;
 
public class ByteArray
{
	public byte[] b;
	private int id;
		
	public ByteArray(byte[] b)
	{
		this.b = b;
		id = -1;
	}
	public ByteArray(byte[] b, int id)
	{
		this.b = b;
		this.id = id;
	}
	/**
	 *Return the data array backed by this class.
	 */
	public byte[] getBytes()
	{
		return b;
	}
	/**
	 *Get a byte at a specific index.
	 */
	public byte getByte(int index)
	{
		if (index > b.length || index < 0) return (byte)0;
		else return b[index];
	}
	/**
	 *Get a range of bytes, from starting point (inclusive)
	 *to start+length (exclusive).
	 *
	 * This method error checks the range; giving an innapropriate
	 * length will result in an array from [start, end) being returned.
	 */
	public byte[] getRange(int start, int length)
	{
		if (start+length >= b.length)
		length = b.length - start;
		
		byte[] out = new byte[length];
		
		for (int i=0; i<length; i++)
			out[i] = b[start+i];
	
		return out;
	}
	/**
	 *See getRange(int start, length)
	 */
	public byte[] getArrayStartingAt(int start)
	{
		byte[] out = new byte[b.length-start];
		for (int i=0; i<out.length; i++)
			out[i] = b[start+i];
		
		return out;
	}
	/**
	 * Get the length of the array.
	 */
	public int getLength()
	{
		return b.length;
	}
	
	/**
	 * Set the id of the ByteArray.
	 */
	 public void setID(int id)
	 {
	 	this.id = id;
	 }
	/**
	 * Get the id of the ByteArray.
	 *
	 * @Return:  An integer of the id.  -1 is the default id.
	 */
	 public int getID()
	 {
	 	return id;
	 }

}