/** This code is under the GPL, viewable at http://www.gnu.org/copyleft/gpl.html
 *  You are free to modify and redistribute the code as you see fit, but three main 
 *  conditions exist for usage of this code:
 *  1.  You may only pass on this code with the same rights and priveleges that were given
 *      to you (In other words, you must redistribute this license)
 *  2.  You may NOT distribute this code for sale or for profit.
 *  3.  There are no warranties about this code.  It is designed not to screw up your computer,
 *      but in the case that it does, the writer cannot be held responsible for losses.
 *
 *  Code author:  flug_m (post comments/questions to http://www.saveourtunes.com/forum/ In future
 *                versions, I'll make available alternative means of contact.)
 *  Code Revision date:  January 27, 2008
 *  Code Revision version: 0.7
 *
 *
 *  The TrafficEvent holds the captured data from the pass-through thread.  
 *
 *  The class holds the following data:
 *
 *    --The captured data from the Pass-through thread,
 *    --The origin of the data (server, client, interrupt, etc)
 *    --The Thread that created the TrafficEvent (if multiple threads exist,
 *      this assists in identifying the owner thread).
 */
 package ourtunes.net;
 
import java.net.*;


public class TrafficEvent
{
	public TrafficEvent(byte[] data, int origin, Thread launcher)
	{
		this.data = data;
		this.origin = origin;
		this.launcher = launcher;
	}
	/**
	 * Return the captured data as a ByteArray (a wrapper class
	 * for the actual data).
	 */
	public ByteArray getByteArray()
	{
		return new ByteArray(data);
	}
	/**
	 * Return the captured data as a byte[].
	 */
	public	byte[] getBytes()
	{
		return data;
	}
	/**
	 * Test if the data came from the server.  If
	 * this method returns false, it doesn't always
	 * mean that the data came from the client.
	 */
	public boolean fromServer()
	{
		return (FROM_SERVER == origin);
	}
	/**
	 * Test if the data came from the client.  If
	 * this method returns false, it doesn't always
	 * mean that the data came from the server.
	 */
	public boolean fromClient()
	{
		return (FROM_CLIENT == origin);
	}
	/**
	 * Get the data origin as a int.  Unless a custom
	 * data origin int is used, the constants of this 
	 * class will indicate the origin of the data.
	 */
	public int getOrigin()
	{
		return origin;
	}
	/**
	 * Convenience method to interrupt the launching
	 * thread.  The reason this method exists is because
	 * of the need to interrupt the pass-through when
	 * the song download is complete.
	 */ 
	public void interruptLaunchingThread()
	{
		launcher.interrupt();
	}

	private byte[] data;
	private int origin;
	private Thread launcher;
	
	/**
	 *public constants to indicate the origin.  
	 *Custom constants can be used, defined here,
	 *or in another class.
	 */
	 
	public static final int INTERRUPT_EVENT = 4;
	public static final int FROM_SERVER = 1;
	public static final int FROM_CLIENT = 2;
	public static final int UNKNOWN = 0;
}

