/** This code is under the GPL, viewable at http://www.gnu.org/copyleft/gpl.html
 *  You are free to modify and redistribute the code as you see fit, but three main 
 *  conditions exist for usage of this code:
 *  1.  You may only pass on this code with the same rights and priveleges that were given
 *      to you (In other words, you must redistribute this license)
 *  2.  You may NOT distribute this code for sale or for profit.
 *  3.  There are no warranties about this code.  It is designed not to screw up your computer,
 *      but in the case that it does, the writer cannot be held responsible for losses.
 *
 *  Code author:  flug_m (post comments/questions to http://www.saveourtunes.com/forum/ In future
 *                versions, I'll make available alternative means of contact.)
 *  Code Revision date:  September 1, 2007
 *  Code Revision version: 0.5
 *
 *
 *  A TrafficListener is used to listen to traffic passing through the java code.
 *
 *  Originally, the design goal was to put traffic listeners on the Host thread,
 *  and all Pass-through threads would inherit the listeners (and thus report
 *  the captured data to them all).  However, iTunes creates new connections for
 *  each song request from the host machine; by keeping with the original intent,
 *  data passing through java would become jumbled up in other threads, which results
 *  in extraneous data being saved into the MP3 files.
 *
 *  In the code's current form, each Pass-Through thread has a unique traffic listener
 *  that it reports to.  This keeps the data passing through the ports segregated, as
 *  it otherwise would be.
 *
 *  Note that it shouldn't take very much effort to re-write the PassThrough class to 
 *  support TrafficListeners in the same manner that swing components support their listeners.
 */
 package ourtunes.net;

public interface TrafficListener
{
	/**
	 * Indicates that a pass-through thread has captured data,
	 * and it is available for use.
	 */
	public void trafficReceived(TrafficEvent event);
}
