/** This code is under the GPL, viewable at http://www.gnu.org/copyleft/gpl.html
 *  You are free to modify and redistribute the code as you see fit, but three main 
 *  conditions exist for usage of this code:
 *  1.  You may only pass on this code with the same rights and priveleges that were given
 *      to you (In other words, you must redistribute this license)
 *  2.  You may NOT distribute this code for sale or for profit.
 *  3.  There are no warranties about this code.  It is designed not to screw up your computer,
 *      but in the case that it does, the writer cannot be held responsible for losses.
 *
 *  Code author:  flug_m (post comments/questions to http://www.saveourtunes.com/forum/ In future
 *                versions, I'll make available alternative means of contact.)
 *  Code Revision date:  September 1, 2007
 *  Code Revision version: 0.5
 *
 *
 * The Operating System interface lists methods that assist in writing configuration data
 * to hard disk, as well as locating the default directory for ourTunes and setting a user-
 * configured download directory.
 * 
 * The class should also provide a means to change the default look and feel of the GUI to
 * the platform specific OS (ie, the windows class should default to the Windows look and feel,
 * while Mac should use the Aqua look and feel). 
 *
 * Known implementing classes:
 *   Windows (Windows XP or Windows Vista)
 *   Mac     (Mac OS X)
 */
 package ourtunes.os;
 
 import java.util.Properties;
 
 
 public interface OS
 {
 	/**
 	 *The default directory for iTunes should be the Music directory
 	 *of the current user's folder.  For example, in windows, the absolute
 	 *pathname should be:
 	 *  C:\Documents And Settings\<user name>\My Documents\My Music\OurTunes\
 	 *
 	 *and for Mac,
 	 *  /<user name>/Music/OurTunes/
 	 *
 	 *Pay attention to the file separators;  a call to
 	 *  System.getProperty("file.seperator") will yield the correct file seperator,
 	 *  regardless of OS the user is on.  Similary, System.getProperty("user.name")
 	 *  returns the user name, and System.getProperty("user.home") returns the user's
 	 *  root directory.
 	 *
 	 * To improve the speed of the method, I'd suggest caching the result after the
 	 * first calculation.
 	 *
 	 *
 	 *@Returns:
 	 *  The String location of the ourTunes default directory.
 	 */
 	public String getDefaultDirectory();
 	
 	/**
 	 * Save the current ourTunes configuration to file.
 	 *
 	 * The properties class provides a mean to output itself to file if you'll
 	 * look at the API.
 	 * 
 	 * Note:  save the file with a ".bin" file extension.
 	 *
 	 * @Returns:
 	 *   true if the file saved;
 	 *   false otherwise.
 	 */
 	public boolean saveConfiguration(Properties p);
 	
 	/**
 	 * Load the configuration file from the default directory.
 	 * Again, look to the API for a means to load the Properties
 	 * from file.
 	 */
 	public Properties loadConfiguration();
 
 	/**Change the Look and feel of the program to match
 	 * the L&F of the OS.
 	 */
 	public void changeLookAndFeel();
 }