/** This code is under the GPL, viewable at http://www.gnu.org/copyleft/gpl.html
 *  You are free to modify and redistribute the code as you see fit, but three main 
 *  conditions exist for usage of this code:
 *  1.  You may only pass on this code with the same rights and priveleges that were given
 *      to you (In other words, you must redistribute this license)
 *  2.  You may NOT distribute this code for sale or for profit.
 *  3.  There are no warranties about this code.  It is designed not to screw up your computer,
 *      but in the case that it does, the writer cannot be held responsible for losses.
 *
 *  Code author:  flug_m (post comments/questions to http://www.saveourtunes.com/forum/ In future
 *                versions, I'll make available alternative means of contact.)
 *  Code Revision date:  September 1, 2007
 *  Code Revision version: 0.5
 *
 *  Changes:
 *  9/3/07  When changeLookAndFeel is called, the components are immediately updated with 
 *          new lookAndFeel.
 */


package ourtunes.os;

import java.util.Properties;
import java.io.*;

public class Windows implements OS
{
	static{
		String user = System.getProperty("user.home");
		String sep = System.getProperty("file.separator");
		
		defaultDirectory = user + sep + "My Documents" + sep + 
			"My Music" + sep + "ourTunes" + sep;
	}
	
	public String getDefaultDirectory()
	{
		File f = new File(defaultDirectory);
		f.mkdirs();
		return defaultDirectory;
	}
	
	public boolean saveConfiguration(Properties p)
	{
		File conf = null;
		try{
			conf = new File(defaultDirectory, "config.xml");
			FileOutputStream fout = new FileOutputStream(conf);
			
			p.storeToXML(fout, null);
			
			
		}
		catch(IOException e)
		{
			if (conf != null)
				conf.delete();
			e.printStackTrace();
			return false;
		}
		
		return true;
	}
	
	public Properties loadConfiguration()
	{
		//We expect the configuration file to be located
		//in the default directory.  If not, then we return
		//null because it wasn't found.
		File conf = new File(defaultDirectory, "config.xml");
		FileInputStream fis = null;
		Properties p = null;
			
		if (!conf.exists()) return null;
		
		try{
		 fis = new FileInputStream(conf);
		 p = new Properties();
		}
		catch(FileNotFoundException f)
		{
			return null;
		}
		 
		try{
			p.loadFromXML(fis);
		}
		catch(IOException e)
		{
			e.printStackTrace();
			conf.delete();
			return null;
		}
		
		return p;
	}
	
	public void changeLookAndFeel()
	{
		try
		{
			String landk = javax.swing.UIManager.getSystemLookAndFeelClassName();
			javax.swing.UIManager.setLookAndFeel(landk);
			javax.swing.SwingUtilities.updateComponentTreeUI(ourtunes.Shared.panel);	
		}
		catch(Exception e)
		{
			//Funny, the look and feel didn't change.
			e.printStackTrace();
		}
	}
	
	
	private static String defaultDirectory;
}